import os
import numpy
from matplotlib import pyplot
from scipy.stats import ttest_rel

pyplot.close('all')

# Auto-detect the data folder
dirname = os.path.dirname( \
    os.path.abspath(__file__))
datadir = os.path.join(dirname, 'data')
# Throw an error if there is no data folder
if not os.path.isdir(datadir):
    raise Exception("Data folder not found.")



# Get the first data file
all_files = os.listdir(datadir)
filename = 'pp0.txt'
filepath = os.path.join(datadir, filename)
# Raise an exception if the file doesn't exist
if not os.path.isfile(filepath):
    raise Exception("File not found!")

# Load the data (FINALLY!)
raw = numpy.loadtxt( \
    filepath, \
    dtype=str, \
    unpack=True, \
    delimiter='\t')

# Convert the raw data into a sensible format
data = {}
# Find the number of columns
ncol = len(raw)
for i in range(ncol):
    varname = raw[i][0]
    values = raw[i][1:]
    try:
        data[varname] = values.astype(float)
    except ValueError:
        data[varname] = values

# Create a dictionary for Boolean arrays
sel = {}
sel['valid'] = data['valid'] == 1
sel['invalid'] = data['valid'] == 0
sel[100] = data['soa'] == 100
sel[900] = data['soa'] == 900

# Loop through conditions
descr = {}
for soa in [100, 900]:
    descr[soa] = {}
    for validity in ['valid', 'invalid']:
        descr[soa][validity] = {}
        # Choose the select0rz
        sel1 = sel[validity]
        sel2 = sel[soa]
        # Compute the proportion correct
        acc_m = numpy.mean( \
            data['correct'][sel1 & sel2])
        # Compute the median RT
        rt_m = numpy.median( \
            data['RT'][sel1 & sel2])
        # Compute the RT standard deviation
        rt_sd = numpy.std( \
            data['RT'][sel1 & sel2])
        # Compute the standard error of the mean
        n = numpy.sum(sel1 & sel2)
        rt_sem = rt_sd / numpy.sqrt(n)
        
        # Store descriptives
        descr[soa][validity]['acc_m'] = \
            acc_m
        descr[soa][validity]['rt_m'] = \
            rt_m
        descr[soa][validity]['rt_sem'] = \
            rt_sem
    # Do a related-samples t test
    sel1 = data['soa'] == soa
    val = data['valid'] == 1
    inval = data['valid'] == 0
    t, p = ttest_rel( \
        data['RT'][sel1 & val], \
        data['RT'][sel1 & inval] \
        )
    # Save the t and p values.
    descr[soa]['t'] = t
    descr[soa]['p'] = p

    print("SOA %d: t=%.2f, p=%.3f" % \
        (soa, t, p))

# Pre-define colours
col = { \
    'valid':   'green', \
    'invalid': 'red'}
# Draw bar graphs
pyplot.figure()
for soa in [100, 900]:
    left = { \
        'valid':    soa - 50, \
        'invalid':  soa}
    for val in ['valid', 'invalid']:
        pyplot.bar(left[val], \
            descr[soa][val]['rt_m'], \
            color=col[val], \
            width=50)
        pyplot.errorbar(left[val] + 25, \
            descr[soa][val]['rt_m'], \
            yerr=descr[soa][val]['rt_sem'], \
            ecolor='black')
pyplot.xlabel("Stimulus onset asynchrony (ms)")
pyplot.ylabel("Response time (ms)")

pyplot.show()