import numpy
from matplotlib import pyplot
from scipy.stats import ttest_rel
from pygazeanalyser.edfreader import \
    read_edf

# Load gaze data file
data = read_edf("ED_pupil.asc", \
    "PUPIL_TRIALSTART", \
    stop="pupdata_stop")

# Plot your first trace
pyplot.plot(data[0]['size'], 'o')
pyplot.show()

# Baseline start
t1, msg = data[0]['events']['msg'][1]
# Data start
t2, msg = data[0]['events']['msg'][2]

# Find the sample numbers (indices) at
# which the baseline started, and at
# which data collection started.
t1i = numpy.where( \
    data[0]['trackertime'] == t1)[0]
t2i = numpy.where( \
    data[0]['trackertime'] == t2)[0]

# Slice the baseline from the signal.
baseline = data[0]['size'][t1i:t2i]
trace = data[0]['size'][t2i:]

# Correct the signal by the baseline.
prop = trace / numpy.median(baseline)

pyplot.figure()
pyplot.plot(prop, '-')
pyplot.show()

# Create dict to store traces
traces = { \
    'black':[], \
    'white':[]}

# Loop through all trials
n_trials = len(data)
for i in range(n_trials):
    # Find out the trial type
    t0, msg = data[i]['events']['msg'][0]
    if 'black' in msg:
        trialtype = 'black'
    elif 'white' in msg:
        trialtype = 'white'
    # Baseline start
    t1, msg = data[i]['events']['msg'][1]
    # Data start
    t2, msg = data[i]['events']['msg'][2]
    
    # Find the sample numbers (indices) at
    # which the baseline started, and at
    # which data collection started.
    t1i = numpy.where( \
        data[i]['trackertime'] == t1)[0]
    t2i = numpy.where( \
        data[i]['trackertime'] == t2)[0]
    
    # Get the baseline trace
    baseline = data[i]['size'][t1i:t2i]
    # Get the pupil trace
    trace = data[i]['size'][t2i:t2i+2000]
    
    # Baseline correct
    trace = trace / numpy.median(baseline)
    
    # Store the trace in the trace dict
    traces[trialtype].append(trace)

# Convert lists to NumPy arrays
traces['white'] = \
    numpy.array(traces['white'])
traces['black'] = \
    numpy.array(traces['black'])

# Compute average traces
avgs = { \
    'black':{}, \
    'white':{}}
# Loop through the conditions.
for con in ['black', 'white']:
    # Calculate the number of trials
    n_trials = len(traces[con])
    # Compute the average trace
    avgs[con]['M'] = numpy.mean( \
        traces[con], axis=0)
    # Compute the standard error of the M
    sd = numpy.std( \
        traces[con], axis=0)
    avgs[con]['SEM'] = \
        sd / numpy.sqrt(n_trials)

# Stats!
t, p = ttest_rel(traces['white'], \
    traces['black'], axis=0)
# Multiple-comparison correction
alpha = 0.05 / len(p)

# Plotting, woohoo!
cols = { \
    'black': '#204a87', \
    'white': '#c4a000'}
pyplot.figure()
for con in ['black', 'white']:
    # Create time values
    x = numpy.arange(0, \
        len(avgs[con]['M']))
    # Plot the mean trace
    pyplot.plot(x, avgs[con]['M'], '-', \
        color=cols[con], label=con)
    # Add the standard error of the M
    y1 = avgs[con]['M'] + \
        avgs[con]['SEM']
    y2 = avgs[con]['M'] - \
        avgs[con]['SEM']
    pyplot.fill_between(x, y1, y2, \
        color=cols[con], alpha=0.3)

# Pretify
pyplot.xlabel("Time (ms)")
pyplot.ylabel("Pupil size (prop. change)")
pyplot.legend(loc='upper left')

pyplot.show()